// Copyright (c) 1984-2007 Sangoma Technologies Inc.
//
// Author: David Rokhvarg <davidr@sangoma.com>
//
// Program for reversing bit order of each byte in a file.
// The input voice file is expected to be 8000 Hz, MuLaw or ALaw.
//
#include <stdio.h>
#include <conio.h>
#include <stddef.h>		//for offsetof()
#include <stdlib.h>

unsigned char wp_brt[256]; 

static void show_usage()
{
	printf("usage  : bit_reverser.exe <input file name>\n");
	printf("example: bit_reverser.exe abc.raw\n");
}

static void generate_bit_rev_table(void)
{
	unsigned char util_char;
	unsigned char misc_status_byte;
	int i;

	/* generate the bit-reversing table for all unsigned characters */
	for(util_char = 0;; util_char ++) {
		misc_status_byte = 0;			/* zero the character to be 'built' */
		/* process all 8 bits of the source byte and generate the */
		for(i = 0; i <= 7; i ++) {
		      	/* corresponding 'bit-flipped' character */
			if(util_char & (1 << i)) {
				misc_status_byte |= (1 << (7 - i));
			}
		}
		/* insert the 'bit-flipped' character into the table 
		 * at the appropriate location */
		wp_brt[util_char] = misc_status_byte; 

		/* exit when all unsigned characters have been processed */
		if(util_char == 0xFF) {
			break;
		}
	}
}

static void bit_swap_a_buffer(unsigned char *data, int len)
{
	int i;
	for (i=0; i < len; i++){
		data[i]=wp_brt[data[i]];
	}
}

void bit_swap_2_files(FILE *pSourceFile, FILE *pDestinationFile)
{
	unsigned char	tmp;
	unsigned int	rc, cnt = 0;

	do
	{
		rc = fread( &tmp, 1, 1, pSourceFile );

		bit_swap_a_buffer(&tmp, 1);

		fwrite( &tmp, 1, 1, pDestinationFile );

		cnt++;

		printf("bitswapped %d bytes\r", cnt);
	}while(rc == 1);
}

int main(int argc, char* argv[])
{
	char	input_file_name[_MAX_PATH];
	FILE	*pInputFile;
	char	output_file_name[_MAX_PATH];
	FILE	*pOutputFile;

	////////////////////////////////////////////////////////////////////////////
	//check, if user provided Input File Name as a command line argument
	if(argc != 2){
		printf("Invalid number of arguments!");
		show_usage();
		return 1;
	}

	generate_bit_rev_table();

	////////////////////////////////////////////////////////////////////////////
	_snprintf(input_file_name, _MAX_PATH, "%s", argv[1]);
	printf("Using input_file_name: '%s'\n", input_file_name);

	pInputFile = fopen( input_file_name, "rb" );
	if(pInputFile == NULL){
		printf("Failed to open '%s'!\n", input_file_name);
		show_usage();
		return 1;
	}

	////////////////////////////////////////////////////////////////////////////
	_snprintf(output_file_name, _MAX_PATH, "out_%s", input_file_name);
	printf("Using output_file_name: '%s'\n", output_file_name);

	pOutputFile = fopen( output_file_name, "wb" );
	if(pOutputFile == NULL){
		printf("Failed to open '%s'!\n", input_file_name);
		show_usage();
		return 1;
	}

	////////////////////////////////////////////////////////////////////////////
	bit_swap_2_files(pInputFile, pOutputFile);
	printf("\nDone.\n");
	return 0;
}

