//////////////////////////////////////////////////////////////////////
// sangoma_interface.h: interface for Sangoma API driver.
//
// Author	:	David Rokhvarg	<davidr@sangoma.com>
//////////////////////////////////////////////////////////////////////

#ifndef SANGOMA_INTERFACE_H
#define SANGOMA_INTERFACE_H

#include <windows.h>
#include <winioctl.h>
#include <stdio.h>
#include <conio.h>
#include <stddef.h>		//for offsetof()


#define __WINDOWS__
#include <wanpipe_defines.h>	//for 'wan_udp_hdr_t'
#include <wanpipe_version.h>	//for DRIVER_VERSION structure
#include <sang_status_defines.h>//return codes
#include <sang_api.h>			//for IOCTL codes
#include <sdla_remora.h>		//Analog card
#include <wanpipe_cfg.h>		//for RBS commands, Media Types (WAN_MEDIA_NONE...)
#include <wanpipe_tdm_api_iface.h>	//for TDMV API
#include <sdla_aft_te1.h>		//for Wanpipe API

#include "sample_cpp.h"


/**
  *@author David Rokhvarg
  */

struct func_protocol {
	int		protocol_id;
	unsigned char   prot_name[10];
	/*
	config_t*	config;
	usage_t*	usage;
	main_t*		main;
	dis_trace_t*	disable_trace;
	main_menu_t*	get_main_menu;
	cmd_menu_t*	get_cmd_menu;
	ft1_cfg_t*	read_FT1_status;
	ft1_cfg_t*	set_FT1_mode;
	*/
	unsigned char	mbox_offset;
};


static struct func_protocol function_lookup[] = {
	{ WANCONFIG_FR,			"fr",		0 }, 
	{ WANCONFIG_MFR,		"fr",		0 },
	{ WANCONFIG_PPP,		"ppp",		0 }, 
	{ WANCONFIG_CHDLC,		"chdlc",	2 }, 
	{ WANCONFIG_ATM,		"atm",		2 }, 
	{ WANCONFIG_AFT,		"aft",		0 },//supported
	{ WANCONFIG_AFT_TE1,	"aft",		0 },//supported
	{ WANCONFIG_AFT_ANALOG,	"aft",		0 },//supported
	{ WANCONFIG_ADSL,		"adsl",		2 }, 
	{ WANCONFIG_SS7,		"ss7",		2 },
	{ WANCONFIG_X25, 		"x25",		0 }, 
	{ WANCONFIG_BITSTRM,	"bitstrm",	0 },
	{ WANCONFIG_BSC, 		"bsc",		0 }, 
	{ WANCONFIG_HDLC, 		"hdlc",		0 }, 
	{ 0,					"N/A",		0 }, 
};

#define DECODE_PROT(prot)((prot==WANCONFIG_FR)?"Frame Relay":	\
			  (prot==WANCONFIG_MFR)?"Frame Relay":				\
	          (prot==WANCONFIG_PPP)?"PPP Point-to-Point":		\
	          (prot==WANCONFIG_CHDLC)?"Cisco HDLC":				\
	          (prot==WANCONFIG_X25)?"X25":						\
	          (prot==WANCONFIG_ADSL)?"ADSL: Eth,IP,PPP/ATM":	\
			  (prot==WANCONFIG_ATM)?"ATM":						\
			  (prot==WANCONFIG_AFT)?"AFT T1/E1":				\
			  (prot==WANCONFIG_AFT_TE1)?"AFT T1/E1":			\
			  (prot==WANCONFIG_AFT_ANALOG)?"AFT Analog":		\
							"Unknown")

class sangoma_interface {

	//////////////////////////////////////////////////////////////////
	HANDLE		hGeneralCommands;
	HANDLE		hRx;
	HANDLE		hTx;

	//////////////////////////////////////////////////////////////////
	//receive stuff
	ULONG		rx_frames_count;
	ULONG		rx_bytes_count;
	//for counting frames with CRC/Abort errors
	ULONG		bad_rx_frames_count;
	HANDLE		hRxThread;
	DWORD		dwRxThreadId; 

	//////////////////////////////////////////////////////////////////
	//transmit stuff
	HANDLE		hTxWaitHandle;
	ULONG		tx_bytes_count;
	ULONG		tx_frames_count;
	UCHAR		tx_test_byte;

	//////////////////////////////////////////////////////////////////
	//IOCTL management structures and variables
	wan_udp_hdr_t	wan_udp;
	int				protocol_cb_size;
	int 			wan_protocol;
	unsigned char	adapter_type;

	wanpipe_tdm_api_cmd_t tdm_api_cmd;

	void PrintRxData(RX_DATA_STRUCT *pRx);
	void PrintTdmApiEvent(RX_DATA_STRUCT *pRx);
	void cleanup();

	wan_cmd_api_t			wanpipe_api_cmd;
	api_tx_hdr_t			*tx_hdr;

	int GetWANConfig( void );
	int get_interface_configuration(if_cfg_t *wanif_conf_ptr);
	unsigned char get_wan_udphdr_data_byte(unsigned char off);
	unsigned char *get_wan_udphdr_data_ptr(unsigned char off);
	unsigned char set_wan_udphdr_data_byte(unsigned char off, unsigned char data);
	int get_fe_type(unsigned char* adapter_type);

	int wanpipe_api_ioctl(wan_cmd_api_t *api_cmd);

	USHORT	DoReadCommand(HANDLE drv, RX_DATA_STRUCT * pRx);
	UCHAR	DoWriteCommand(HANDLE drv, TX_DATA_STRUCT * pTx);
	UCHAR	DoSetIdleTxBufferCommand(HANDLE drv, TX_DATA_STRUCT *pTx);

	unsigned char wp_brt[256];
	void generate_bit_rev_table(void);

	char is_rbs_monitoring_enabled;

public:
	//////////////////////////////////////////////////////////////////
	//data
	char					terminate_tx_rx_threads;
	RX_DATA_STRUCT			rx_data;
	callback_functions_t	callback_functions;
	unsigned char			last_hook_state_event;

	TX_DATA_STRUCT			tx_data;

	char					device_name[DEV_NAME_LEN];	
	if_cfg_t				wanif_conf_struct;

	//////////////////////////////////////////////////////////////////
	//methods
	sangoma_interface(int wanpipe_number, int interface_number);
	~sangoma_interface();

	int DoManagementCommand(HANDLE drv, wan_udp_hdr_t* packet);
#define DO_COMMAND(wan_udp)	DoManagementCommand(hGeneralCommands, &wan_udp);

	int init(callback_functions_t *callback_functions_ptr);
	int run();
	int stop();

	void bit_swap_a_buffer(unsigned char *data, int len);

	void read_te1_56k_stat(void);
	void set_lb_modes(unsigned char type, unsigned char mode);
	
	void operational_stats (void);
	void flush_operational_stats (void);

	void enable_rbs_monitoring();
	char get_rbs(rbs_management_t *rbs_management_ptr);
	char set_rbs(rbs_management_t *rbs_management_ptr);

	int sangoma_tdm_enable_rbs_events(int polls_per_second);
	int sangoma_tdm_disable_rbs_events();

	void set_idle_tx_data_buffer(unsigned char idle_tx_test_byte);
	int get_open_handles_counter();

	HANDLE open_api_device(char *api_device_name);

	unsigned char get_adapter_type()
	{
		return adapter_type;
	}

	unsigned int get_sub_media()
	{
		return wanif_conf_struct.sub_media;
	}

	void read_api_driver_version (void);

	UCHAR transmit(TX_DATA_STRUCT *pTx)
	{
		UCHAR rc = DoWriteCommand(hTx, pTx);
		if(rc){
			//error
			printf("DoWriteCommand() failed!! Check messages log.\n");
			return rc;
		}

		//check that frame was transmitted
		switch(pTx->api_header.operation_status)
		{
		case SANG_STATUS_SUCCESS:
			//printf("Frame queued for transmission.\n");
#if 0
			//Some useful information about API's internal transmit queue
			//is available after each successful transmission:
			printf("max_tx_queue_length: %d current_number_of_frames_in_tx_queue: %d\n",  
				pTx->api_header.tx_h.max_tx_queue_length,
				pTx->api_header.tx_h.current_number_of_frames_in_tx_queue);
#endif
			break;

		case SANG_STATUS_TX_TIMEOUT:
			//error
			//It means that tx buffers are constanly full,
			//usually because the line is disconnected
			printf("****** SANG_STATUS_TX_TIMEOUT ******\n");
			//Check messages log or look at statistics.
			break;
					
		case SANG_STATUS_TX_DATA_TOO_LONG:
			//Possible causes:
			//1. User specified pTx->api_header.data_length is too big.
			//2. Attempt to transmit data longer than the pre-configured maximum.
			//   Maximum length is set in 'Interface Properties',
			//   in the 'Device Manager'.
			//Check messages log.
			printf("****** SANG_STATUS_TX_DATA_TOO_LONG ******\n");
			break;
					
		case SANG_STATUS_TX_DATA_TOO_SHORT:
			//Possible cause:
			//1. Minimum is 1 byte.
					printf("****** SANG_STATUS_TX_DATA_TOO_SHORT ******\n");
			break;

		case SANG_STATUS_LINE_DISCONNECTED:
			//Front end monitoring is enabled and Line is
			//in disconnected state.
			//Check the T1/E1 line is in "Connected" state,
			//alse check the Alarms and the message log.
			printf("****** SANG_STATUS_LINE_DISCONNECTED ******\n");
			break;

		default:
			printf("Unknown return code (0x%X) on transmission!\n",
				pTx->api_header.operation_status);
			break;
		}//switch()

		return rc;
	}

	USHORT receive(RX_DATA_STRUCT *pRx)
	{
		return DoReadCommand(hRx, pRx);
	}

	UCHAR DoSetSharedEventCommand(PREGISTER_EVENT event)
	{
		DWORD ln;
	
		if(DeviceIoControl(
			hRx,
			IoctlSetSharedEvent,
			(LPVOID)event,
			sizeof(REGISTER_EVENT),
			(LPVOID)event,
			sizeof(REGISTER_EVENT),
			(LPDWORD)(&ln),
			(LPOVERLAPPED)NULL
						) == FALSE){
			//check messages log
			return 1;
		}else{
			return 0;
		}
	}

	//////////////////////////////////////////////////////////////////
	//TDM API calls
	int sangoma_tdm_enable_rxhook_events();
	int sangoma_tdm_disable_rxhook_events();

	/* DTMF Detection on A200 Analog card (SLIC) chip */
	int sangoma_tdm_enable_rm_dtmf_events();
	int sangoma_tdm_disable_rm_dtmf_events();

	/* DTMF Detection on Octasic chip */
	int sangoma_tdm_enable_dtmf_events(int channel);
	int sangoma_tdm_disable_dtmf_events(int channel);

	int sangoma_tdm_enable_ring_detect_events();
	int sangoma_tdm_disable_ring_detect_events(); 

	int sangoma_tdm_enable_ring_trip_detect_events(); 
	int sangoma_tdm_disable_ring_trip_detect_events();

	int sangoma_tdm_enable_ring_events();
	int sangoma_tdm_disable_ring_events();

	int sangoma_tdm_txsig_onhook();
	int sangoma_tdm_txsig_offhook();

	int sangoma_tdm_enable_tone_events(int tone_id);
	int sangoma_tdm_disable_tone_events();
	//////////////////////////////////////////////////////////////////

	int tdmv_api_ioctl(wanpipe_tdm_api_cmd_t *api_cmd);

	int reset_interface_state();

	int start_ring_tone();
	int stop_ring_tone();

	int start_congestion_tone();
	int stop_congestion_tone();

	int start_busy_tone();
	int stop_busy_tone();

	int stop_all_tones();

	int start_dial_tone();
	int stop_dial_tone();

	int start_ringing_phone();
	int stop_ringing_phone();

	int fxo_go_off_hook();
	int fxo_go_on_hook();

};

#endif
